@@ -9,16 +9,19 @@ import logging
 PY2 = sys.version_info[0] == 2
 
 if not PY2:
-    iteritems = lambda d: iter(d.items())
-    unicode = str
+    iteritems = lambda d: d.items()
     text_type = str
+    binary_type = bytes
+    long_type = float
 else:
     iteritems = lambda d: d.iteritems()
-    unicode = unicode
-    text_type = unicode
+    text_type = unicode  # noqa
+    binary_type = str
+    long_type = long  # noqa
 
 logger = logging.getLogger(__name__)
 
+
 class WebargsError(Exception):
     """Base class for all webargs-related errors."""
     pass
@@ -37,7 +40,7 @@ class ValidationError(WebargsError):
         Store status_code and additonal data.
     """
     def __init__(self, error, status_code=400, **data):
-        self.message = unicode(error)
+        self.message = text_type(error)
         self.status_code = status_code
         self.data = data
         super(ValidationError, self).__init__(self.message)
@@ -57,14 +60,15 @@ def _callable_or_raise(obj):
     else:
         return obj
 
-def _ensure_list_of_callables(validate):
-    if validate:
-        if isinstance(validate, (list, tuple)):
-            validators = validate
-        elif callable(validate):
-            validators = [validate]
+
+def _ensure_list_of_callables(obj):
+    if obj:
+        if isinstance(obj, (list, tuple)):
+            validators = obj
+        elif callable(obj):
+            validators = [obj]
         else:
-            raise ValueError('validate must be a callable or list of callables.')
+            raise ValueError('{0!r} is not a callable or list of callables.'.format(obj))
     else:
         validators = []
     return validators
@@ -111,11 +115,30 @@ def noop(x):
     return x
 
 
+__type_map__ = {
+    list: 'array',
+    tuple: 'array',
+    set: 'array',
+    bool: 'boolean',
+    int: 'integer',
+    float: 'number',
+    long_type: 'number',
+    type(None): 'null',
+    dict: 'object',
+    text_type: 'string',
+    binary_type: 'string',
+}
+
+__non_nullable_types__ = set([list, tuple, set, dict, bool])
+
+
 class Arg(object):
     """A request argument.
 
-    :param type type\_: Value type. Will try to convert the passed in value to this
-        type. If ``None``, no type conversion will be performed.
+    :param type\_: Value type or nested `Arg` dictionary. If the former,
+        the parsed value will be coerced this type. If the latter, the parsed
+        value will be validated and converted according to the nested `Arg` dict.
+        If ``None``, no type conversion will be performed.
     :param default: Default value for the argument. Used if the value is not found
         on the request. May be a callable.
     :param bool required: If ``True``, the :meth:`Parser.handle_error` method will be
@@ -123,8 +146,8 @@ class Arg(object):
     :param callable validate: Callable (function or object with ``__call__`` method
         defined) or list of callables. used for custom validation. A validator may return
         a boolean or raise a :exc:`ValidationError`.
-    :param callable use: Function used for converting or pre-processing the value.
-        Defaults to noop. Example: ``use=lambda s: s.lower()``
+    :param callable use: Function or list of functions used for converting
+        or pre-processing the value.  Defaults to noop. Example: ``use=lambda s: s.lower()``
     :param bool multiple: Return a list of values for the argument. Useful for
         querystrings or forms that pass multiple values to the same parameter,
         e.g. ``/?name=foo&name=bar``
@@ -142,14 +165,21 @@ class Arg(object):
     def __init__(self, type_=None, default=None, required=False,
                  validate=None, use=None, multiple=False, error=None,
                  allow_missing=False, target=None, source=None, **metadata):
-        self.type = type_ or noop  # default to no type conversion
+        if isinstance(type_, dict):
+            self.type = dict
+            self._nested_args = type_
+            self._has_nesting = True
+        else:
+            self.type = type_ or noop  # default to no type conversion
+            self._nested_args = None
+            self._has_nesting = False
         if multiple and default is None:
             self.default = []
         else:
             self.default = default
         self.required = required
         self.validators = _ensure_list_of_callables(validate)
-        self.use = _callable_or_raise(use) or noop
+        self.use_funcs = _ensure_list_of_callables(use)
         self.error = error
         self.multiple = multiple
         if required and allow_missing:
@@ -163,14 +193,24 @@ class Arg(object):
         return ('<webargs.core.Arg(type_={self.type}, default={self.default!r}, '
                 'required={self.required})>').format(self=self)
 
-    def _validate(self, value):
+    def _validate(self, name, value):
         """Perform conversion and validation on ``value``."""
         ret = value
-        # First convert the value
+        for func in self.use_funcs:
+            ret = func(ret)
+
+        msg = 'Expected type "{0}" for {1}, got "{2}"'.format(
+            __type_map__.get(self.type, self.type.__name__), name,
+            __type_map__.get(type(ret), type(ret).__name__)
+        )
+        if ret is None and self.type in __non_nullable_types__:
+            raise ValidationError(self.error or msg)
+
         try:
-            ret = self.type(self.use(value))
-        except ValueError as error:
-            raise ValidationError(self.error or error)
+            ret = self.type(ret)
+        except (ValueError, TypeError):
+            raise ValidationError(self.error or msg)
+
         # Then call validation functions
         for validator in self.validators:
             if validator(ret) is False:
@@ -178,9 +218,22 @@ class Arg(object):
                     validator.__name__, ret
                 )
                 raise ValidationError(self.error or msg)
+
+        if self._has_nesting:
+            # Recurse into nested argdict
+            for key, nested_arg in iteritems(self._nested_args):
+                try:
+                    val = ret[key]
+                except KeyError:
+                    if nested_arg.required:
+                        raise ValidationError(
+                            'Required parameter "{0}" not found.'.format(key)
+                        )
+                else:
+                    ret[key] = nested_arg.validated(key, val)
         return ret
 
-    def validated(self, value):
+    def validated(self, name, value):
         """Convert and validate the given value according to the ``type_``,
         ``use``, and ``validate`` attributes.
 
@@ -190,10 +243,10 @@ class Arg(object):
         # Prevent "missing" placeholder from getting converted
         if value is Missing:
             return value
-        if self.multiple and isinstance(value, list):
-            return [self._validate(each) for each in value]
+        if self.multiple and isinstance(value, (list, tuple)):
+            return [self._validate(name, each) for each in value]
         else:
-            return self._validate(value)
+            return self._validate(name, value)
 
 
 class Parser(object):
@@ -207,10 +260,11 @@ class Parser(object):
     :param callable error_handler: Custom error handler function.
     :param str error: Custom error message to use if validation fails.
     """
+
     DEFAULT_TARGETS = ('querystring', 'form', 'json',)
 
     #: Maps target => method name
-    TARGET_MAP = {
+    __target_map__ = {
         'json': 'parse_json',
         'querystring': 'parse_querystring',
         'query': 'parse_querystring',
@@ -224,6 +278,8 @@ class Parser(object):
         self.targets = targets or self.DEFAULT_TARGETS
         self.error_callback = _callable_or_raise(error_handler)
         self.error = error
+        #: A short-lived cache to store results from processing request bodies.
+        self._cache = {}
 
     def _validated_targets(self, targets):
         """Ensure that the given targets argument is valid.
@@ -232,7 +288,7 @@ class Parser(object):
         """
         # The set difference between the given targets and the available targets
         # will be the set of invalid targets
-        valid_targets = set(self.TARGET_MAP.keys())
+        valid_targets = set(self.__target_map__.keys())
         given = set(targets)
         invalid_targets = given - valid_targets
         if len(invalid_targets):
@@ -243,7 +299,7 @@ class Parser(object):
     def _get_value(self, name, argobj, req, target):
         # Parsing function to call
         # May be a method name (str) or a function
-        func = self.TARGET_MAP.get(target)
+        func = self.__target_map__.get(target)
         if func:
             if inspect.isfunction(func):
                 function = func
@@ -280,7 +336,7 @@ class Parser(object):
                 continue
             # Found the value; validate and return it
             if value is not Missing:
-                return argobj.validated(value)
+                return argobj.validated(name, value)
         if value is Missing:
             if argobj.default is not None:
                 if callable(argobj.default):
@@ -288,7 +344,7 @@ class Parser(object):
                 else:
                     value = argobj.default
             if argobj.required:
-                raise ValidationError('Required parameter {0!r} not found.'.format(name))
+                raise ValidationError('Required parameter "{0}" not found.'.format(name))
         return value
 
     def parse(self, argmap, req, targets=None, validate=None, force_all=False):
@@ -306,8 +362,8 @@ class Parser(object):
          :return: A dictionary of parsed arguments
         """
         validators = _ensure_list_of_callables(validate)
+        parsed = {}
         try:
-            parsed = {}
             for argname, argobj in iteritems(argmap):
                 parsed_value = self.parse_arg(argname, argobj, req,
                     targets=targets or self.targets)
@@ -327,12 +383,19 @@ class Parser(object):
                         validator.__name__, parsed
                     )
                     raise ValidationError(self.error or msg)
-            return parsed
-        except Exception as error:
+        except ValidationError as error:
             if self.error_callback:
                 self.error_callback(error)
             else:
                 self.handle_error(error)
+        finally:
+            self.clear_cache()
+        return parsed
+
+    def clear_cache(self):
+        """Invalidate the parser's cache."""
+        self._cache = {}
+        return None
 
     def use_args(self, argmap, req=None, targets=None, as_kwargs=False, validate=None):
         """Decorator that injects parsed arguments into a view function or method.
@@ -407,7 +470,7 @@ class Parser(object):
         :param str name: The name of the target to register.
         """
         def decorator(func):
-            self.TARGET_MAP[name] = func
+            self.__target_map__[name] = func
             return func
         return decorator
 
