@@ -30,6 +30,17 @@ def parse_json(s):
         s = s.decode('utf-8')
     return json.loads(s)
 
+def parse_json_body(req):
+    """Return the decoded JSON body from the request."""
+    content_type = req.headers.get('Content-Type')
+    if content_type and 'application/json' in req.headers.get('Content-Type'):
+        try:
+            return parse_json(req.body)
+        except (TypeError, ValueError):
+            pass
+    return {}
+
+
 def get_value(d, name, multiple):
     """Handle gets from 'multidicts' made of lists
 
@@ -54,7 +65,10 @@ class TornadoParser(core.Parser):
 
     def parse_json(self, req, name, arg):
         """Pull a json value from the request."""
-        return get_value(self.json, name, arg.multiple)
+        json_body = self._cache.get('json')
+        if json_body is None:
+            self._cache['json'] = parse_json_body(req)
+        return get_value(self._cache['json'], name, arg.multiple)
 
     def parse_querystring(self, req, name, arg):
         """Pull a querystring value from the request."""
@@ -90,24 +104,6 @@ class TornadoParser(core.Parser):
         data = getattr(error, 'data', {})
         raise tornado.web.HTTPError(status_code, error.args[0], **data)
 
-    def _parse_json_body(self, req):
-        content_type = req.headers.get('Content-Type')
-        if content_type and 'application/json' in req.headers.get('Content-Type'):
-            try:
-                self.json = parse_json(req.body)
-            except (TypeError, ValueError):
-                self.json = {}
-        else:
-            self.json = {}
-
-    def parse(self, argmap, req, *args, **kwargs):
-        """Parses the request using the given arguments map.
-
-        Initializes :attr:`json` attribute.
-        """
-        self._parse_json_body(req)
-        return super(TornadoParser, self).parse(argmap, req, *args, **kwargs)
-
     def use_args(self, argmap, req=None, targets=core.Parser.DEFAULT_TARGETS,
                  as_kwargs=False, validate=None):
         """Decorator that injects parsed arguments into a view function or method.
